@@ -354,7 +354,7 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
     char *po, *pi, *pc = NULL;
     npy_intp **edge_offsets = NULL, **data_offsets = NULL, filter_size;
     npy_intp ftmp[MAXDIM], *fcoordinates = NULL, *foffsets = NULL;
-    npy_intp cstride = 0, kk, hh, ll, jj, *idxs = NULL;
+    npy_intp cstride = 0, kk, hh, ll, jj;
     npy_intp size;
     double **splvals = NULL, icoor[MAXDIM];
     npy_intp idimensions[MAXDIM], istrides[MAXDIM];
@@ -416,11 +416,6 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
     filter_size = 1;
     for(jj = 0; jj < irank; jj++)
         filter_size *= order + 1;
-    idxs = (npy_intp*)malloc(filter_size * sizeof(idxs));
-    if (!idxs) {
-        PyErr_NoMemory();
-        goto exit;
-    }
 
     /* initialize output iterator: */
     if (!NI_InitPointIterator(output, &io))
@@ -462,7 +457,8 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
         size *= output->dimensions[qq];
     for(kk = 0; kk < size; kk++) {
         double t = 0.0;
-        int constant = 0, edge = 0, offset = 0;
+        int constant = 0, edge = 0;
+        npy_intp offset = 0;
         if (map) {
             /* call mappint functions: */
             if (!map(io.coordinates, icoor, orank, irank, map_data)) {
@@ -509,11 +505,11 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
             double cc = map_coordinate(icoor[hh], idimensions[hh], mode);
             if (cc > -1.0) {
                 /* find the filter location along this axis: */
-                int start;
+                npy_intp start;
                 if (order & 1) {
-                    start = (int)floor(cc) - order / 2;
+                    start = (npy_intp)floor(cc) - order / 2;
                 } else {
-                    start = (int)floor(cc + 0.5) - order / 2;
+                    start = (npy_intp)floor(cc + 0.5) - order / 2;
                 }
                 /* get the offset to the start of the filter: */
                 offset += istrides[hh] * start;
@@ -522,12 +518,12 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
                     edge = 1;
                     edge_offsets[hh] = data_offsets[hh];
                     for(ll = 0; ll <= order; ll++) {
-                        int idx = start + ll;
-                        int len = idimensions[hh];
+                        npy_intp idx = start + ll;
+                        npy_intp len = idimensions[hh];
                         if (len <= 1) {
                             idx = 0;
                         } else {
-                            int s2 = 2 * len - 2;
+                            npy_intp s2 = 2 * len - 2;
                             if (idx < 0) {
                                 idx = s2 * (int)(-idx / s2) + idx;
                                 idx = idx <= 1 - len ? idx + s2 : -idx;
@@ -554,9 +550,13 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
 
         if (!constant) {
             npy_intp *ff = fcoordinates;
+            const int type_num = NI_NormalizeType(input->descr->type_num);
+            t = 0.0;
             for(hh = 0; hh < filter_size; hh++) {
-                int idx = 0;
-                if (edge) {
+                double coeff = 0.0;
+                npy_intp idx = 0;
+
+                if (NI_UNLIKELY(edge)) {
                     for(ll = 0; ll < irank; ll++) {
                         if (edge_offsets[ll])
                             idx += edge_offsets[ll][ff[ll]];
@@ -567,29 +567,20 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
                     idx = foffsets[hh];
                 }
                 idx += offset;
-                idxs[hh] = idx;
-                ff += irank;
-            }
-        }
-        if (!constant) {
-            npy_intp *ff = fcoordinates;
-            t = 0.0;
-            for(hh = 0; hh < filter_size; hh++) {
-                double coeff = 0.0;
-                switch (NI_NormalizeType(input->descr->type_num)) {
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Bool);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt8);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt16);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt32);
+                switch (type_num) {
+                    CASE_INTERP_COEFF(coeff, pi, idx, Bool);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt8);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt16);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt32);
 #if HAS_UINT64
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt64);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt64);
 #endif
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int8);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int16);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int32);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int64);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Float32);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Float64);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int8);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int16);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int32);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int64);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Float32);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Float64);
                 default:
                     PyErr_SetString(PyExc_RuntimeError,
                                                     "data type not supported");
@@ -652,8 +643,6 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
         free(foffsets);
     if (fcoordinates)
         free(fcoordinates);
-    if (idxs)
-        free(idxs);
     return PyErr_Occurred() ? 0 : 1;
 }
 
@@ -665,7 +654,7 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
     npy_intp **zeros = NULL, **offsets = NULL, ***edge_offsets = NULL;
     npy_intp ftmp[MAXDIM], *fcoordinates = NULL, *foffsets = NULL;
     npy_intp jj, hh, kk, filter_size, odimensions[MAXDIM];
-    npy_intp idimensions[MAXDIM], istrides[MAXDIM], *idxs = NULL;
+    npy_intp idimensions[MAXDIM], istrides[MAXDIM];
     npy_intp size;
     double ***splvals = NULL;
     NI_Iterator io;
@@ -742,13 +731,13 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
                 cc *= zoom;
             cc = map_coordinate(cc, idimensions[jj], mode);
             if (cc > -1.0) {
-                int start;
+                npy_intp start;
                 if (zeros && zeros[jj])
                     zeros[jj][kk] = 0;
                 if (order & 1) {
-                    start = (int)floor(cc) - order / 2;
+                    start = (npy_intp)floor(cc) - order / 2;
                 } else {
-                    start = (int)floor(cc + 0.5) - order / 2;
+                    start = (npy_intp)floor(cc + 0.5) - order / 2;
                 }
                 offsets[jj][kk] = istrides[jj] * start;
                 if (start < 0 || start + order >= idimensions[jj]) {
@@ -758,17 +747,17 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
                         goto exit;
                     }
                     for(hh = 0; hh <= order; hh++) {
-                        int idx = start + hh;
-                         int len = idimensions[jj];
+                        npy_intp idx = start + hh;
+                        npy_intp len = idimensions[jj];
                         if (len <= 1) {
                             idx = 0;
                         } else {
-                            int s2 = 2 * len - 2;
+                            npy_intp s2 = 2 * len - 2;
                             if (idx < 0) {
-                                idx = s2 * (int)(-idx / s2) + idx;
+                                idx = s2 * (npy_intp)(-idx / s2) + idx;
                                 idx = idx <= 1 - len ? idx + s2 : -idx;
                             } else if (idx >= len) {
-                                idx -= s2 * (int)(idx / s2);
+                                idx -= s2 * (npy_intp)(idx / s2);
                                 if (idx >= len)
                                     idx = s2 - idx;
                             }
@@ -793,11 +782,6 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
     filter_size = 1;
     for(jj = 0; jj < rank; jj++)
         filter_size *= order + 1;
-    idxs = (npy_intp*)malloc(filter_size * sizeof(idxs));
-    if (!idxs) {
-        PyErr_NoMemory();
-        goto exit;
-    }
 
     if (!NI_InitPointIterator(output, &io))
         goto exit;
@@ -836,7 +820,7 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
         size *= output->dimensions[qq];
     for(kk = 0; kk < size; kk++) {
         double t = 0.0;
-        int edge = 0, oo = 0, zero = 0;
+        npy_intp edge = 0, oo = 0, zero = 0;
 
         for(hh = 0; hh < rank; hh++) {
             if (zeros && zeros[hh][io.coordinates[hh]]) {
@@ -851,9 +835,13 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
 
         if (!zero) {
             npy_intp *ff = fcoordinates;
+            const int type_num = NI_NormalizeType(input->descr->type_num);
+            t = 0.0;
             for(hh = 0; hh < filter_size; hh++) {
-                int idx = 0;
-                if (edge) {
+                npy_intp idx = 0;
+                double coeff = 0.0;
+
+                if (NI_UNLIKELY(edge)) {
                         /* use precalculated edge offsets: */
                     for(jj = 0; jj < rank; jj++) {
                         if (edge_offsets[jj][io.coordinates[jj]])
@@ -866,29 +854,20 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
                     /* use normal offsets: */
                     idx += oo + foffsets[hh];
                 }
-                idxs[hh] = idx;
-                ff += rank;
-            }
-        }
-        if (!zero) {
-            npy_intp *ff = fcoordinates;
-            t = 0.0;
-            for(hh = 0; hh < filter_size; hh++) {
-                double coeff = 0.0;
-                switch (NI_NormalizeType(input->descr->type_num)) {
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Bool);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt8);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt16);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt32);
+                switch (type_num) {
+                    CASE_INTERP_COEFF(coeff, pi, idx, Bool);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt8);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt16);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt32);
 #if HAS_UINT64
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt64);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt64);
 #endif
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int8);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int16);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int32);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int64);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Float32);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Float64);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int8);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int16);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int32);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int64);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Float32);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Float64);
                 default:
                     PyErr_SetString(PyExc_RuntimeError,
                                                     "data type not supported");
@@ -969,7 +948,5 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
         free(foffsets);
     if (fcoordinates)
         free(fcoordinates);
-    if (idxs)
-        free(idxs);
     return PyErr_Occurred() ? 0 : 1;
 }
