@@ -4,12 +4,12 @@ from decimal import Decimal
 
 from numpy.testing import (TestCase, run_module_suite, assert_equal,
     assert_almost_equal, assert_array_equal, assert_array_almost_equal,
-    assert_raises, assert_allclose, assert_)
+    assert_raises, assert_allclose, assert_, dec)
 
 import scipy.signal as signal
 from scipy.signal import (correlate, convolve, convolve2d, fftconvolve,
      hilbert, hilbert2, lfilter, lfilter_zi, filtfilt, butter, tf2zpk,
-     invres)
+     invres, vectorstrength, signaltools)
 
 
 from numpy import array, arange
@@ -93,6 +93,13 @@ class _TestConvolve2d(TestCase):
         h = array([[62,80,98,116,134]])
         assert_array_equal(g, h)
 
+    def test_valid_mode_complx(self):
+        e = [[2,3,4,5,6,7,8], [4,5,6,7,8,9,10]]
+        f = np.array([[1,2,3], [3,4,5]], dtype=np.complex) + 1j
+        g = convolve2d(e, f, 'valid')
+        h = array([[62.+24.j, 80.+30.j, 98.+36.j, 116.+42.j, 134.+48.j]])
+        assert_array_almost_equal(g, h)
+
     def test_fillvalue(self):
         a = [[1,2,3],[3,4,5]]
         b = [[2,3,4],[4,5,6]]
@@ -232,6 +239,121 @@ class TestFFTConvolve(TestCase):
         d = np.convolve(a, b, 'full')
         assert_(np.allclose(c, d, rtol=1e-10))
 
+    @dec.slow
+    def test_many_sizes(self):
+        np.random.seed(1234)
+
+        def ns():
+            for j in range(1, 100):
+                yield j 
+            for j in range(1000, 1500):
+                yield j
+            for k in range(50):
+                yield np.random.randint(1001, 10000)
+
+        for n in ns():
+            msg = 'n=%d' % (n,)
+            a = np.random.rand(n) + 1j*np.random.rand(n)
+            b = np.random.rand(n) + 1j*np.random.rand(n)
+            c = fftconvolve(a, b, 'full')
+            d = np.convolve(a, b, 'full')
+            assert_allclose(c, d, atol=1e-10, err_msg=msg)
+
+    def test_next_regular(self):
+        np.random.seed(1234)
+
+        def ns():
+            for j in range(1, 1000):
+                yield j
+            yield 2**5 * 3**5 * 4**5 + 1
+
+        for n in ns():
+            m = signaltools._next_regular(n)
+            msg = "n=%d, m=%d" % (n, m)
+
+            assert_(m >= n, msg)
+
+            # check regularity
+            k = m
+            for d in [2, 3, 5]:
+                while True:
+                    a, b = divmod(k, d)
+                    if b == 0:
+                        k = a
+                    else:
+                        break
+            assert_equal(k, 1, err_msg=msg)
+
+    def test_next_regular_strict(self):
+        hams = {
+            1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6, 7: 8, 8: 8, 14: 15, 15: 15,
+            16: 16, 17: 18, 1021: 1024, 1536: 1536, 51200000: 51200000, 
+            510183360: 510183360, 510183360+1: 512000000, 511000000: 512000000,
+            854296875: 854296875, 854296875+1: 859963392, 
+            196608000000: 196608000000, 196608000000+1: 196830000000,
+            8789062500000: 8789062500000, 8789062500000+1: 8796093022208,
+            206391214080000: 206391214080000, 206391214080000+1: 206624260800000,
+            470184984576000: 470184984576000, 470184984576000+1: 470715894135000,
+            7222041363087360: 7222041363087360, 
+            7222041363087360+1: 7230196133913600,
+            # power of 5    5**23
+            11920928955078125: 11920928955078125,
+            11920928955078125-1: 11920928955078125,
+            # power of 3    3**34
+            16677181699666569: 16677181699666569,
+            16677181699666569-1: 16677181699666569,
+            # power of 2   2**54
+            18014398509481984: 18014398509481984,
+            18014398509481984-1: 18014398509481984,
+            # above this, int(ceil(n)) == int(ceil(n+1))
+            19200000000000000: 19200000000000000,
+            19200000000000000+1: 19221679687500000,
+            288230376151711744:   288230376151711744,
+            288230376151711744+1: 288325195312500000,
+            288325195312500000-1: 288325195312500000,
+            288325195312500000:   288325195312500000,
+            288325195312500000+1: 288555831593533440,
+            # power of 3    3**83
+            3990838394187339929534246675572349035227-1:
+                3990838394187339929534246675572349035227,
+            3990838394187339929534246675572349035227:
+                3990838394187339929534246675572349035227,
+            # power of 2     2**135
+            43556142965880123323311949751266331066368-1:
+                43556142965880123323311949751266331066368,
+            43556142965880123323311949751266331066368:
+                43556142965880123323311949751266331066368,
+            # power of 5      5**57
+            6938893903907228377647697925567626953125-1:
+                6938893903907228377647697925567626953125,
+            6938893903907228377647697925567626953125:
+                6938893903907228377647697925567626953125,
+            # http://www.drdobbs.com/228700538
+            # 2**96 * 3**1 * 5**13
+            290142196707511001929482240000000000000-1:
+                290142196707511001929482240000000000000,
+            290142196707511001929482240000000000000:
+                290142196707511001929482240000000000000,
+            290142196707511001929482240000000000000+1:
+                290237644800000000000000000000000000000,
+            # 2**36 * 3**69 * 5**7
+            4479571262811807241115438439905203543080960000000-1:
+                4479571262811807241115438439905203543080960000000,
+            4479571262811807241115438439905203543080960000000:
+                4479571262811807241115438439905203543080960000000,
+            4479571262811807241115438439905203543080960000000+1:
+                4480327901140333639941336854183943340032000000000,
+            # 2**37 * 3**44 * 5**42
+            30774090693237851027531250000000000000000000000000000000000000-1:
+                30774090693237851027531250000000000000000000000000000000000000,
+            30774090693237851027531250000000000000000000000000000000000000:
+                30774090693237851027531250000000000000000000000000000000000000,
+            30774090693237851027531250000000000000000000000000000000000000+1:
+                30778180617309082445871527002041377406962596539492679680000000,
+        }
+        for x, y in hams.items():
+            assert_equal(signaltools._next_regular(x), y)
+
 
 class TestMedFilt(TestCase):
     def test_basic(self):
@@ -850,5 +972,153 @@ class TestPartialFractionExpansion(TestCase):
         assert_raises(ValueError, invres, r, p, k, rtype='median')
 
 
+class TestVectorstrength(TestCase):
+    def test_single_1dperiod(self):
+        events = np.array([.5])
+        period = 5.
+        targ_strength = 1.
+        targ_phase = .1
+
+        strength, phase = vectorstrength(events, period)
+
+        assert_equal(strength.ndim, 0)
+        assert_equal(phase.ndim, 0)
+        assert_almost_equal(strength, targ_strength)
+        assert_almost_equal(phase, 2*np.pi*targ_phase)
+
+    def test_single_2dperiod(self):
+        events = np.array([.5])
+        period = [1, 2, 5.]
+        targ_strength = [1.]*3
+        targ_phase = np.array([.5, .25, .1])
+
+        strength, phase = vectorstrength(events, period)
+
+        assert_equal(strength.ndim, 1)
+        assert_equal(phase.ndim, 1)
+        assert_array_almost_equal(strength, targ_strength)
+        assert_almost_equal(phase, 2*np.pi*targ_phase)
+
+    def test_equal_1dperiod(self):
+        events = np.array([.25, .25, .25, .25, .25, .25])
+        period = 2
+        targ_strength = 1.
+        targ_phase = .125
+
+        strength, phase = vectorstrength(events, period)
+
+        assert_equal(strength.ndim, 0)
+        assert_equal(phase.ndim, 0)
+        assert_almost_equal(strength, targ_strength)
+        assert_almost_equal(phase, 2*np.pi*targ_phase)
+
+    def test_equal_2dperiod(self):
+        events = np.array([.25, .25, .25, .25, .25, .25])
+        period = [1, 2,]
+        targ_strength = [1.]*2
+        targ_phase = np.array([.25, .125])
+
+        strength, phase = vectorstrength(events, period)
+
+        assert_equal(strength.ndim, 1)
+        assert_equal(phase.ndim, 1)
+        assert_almost_equal(strength, targ_strength)
+        assert_almost_equal(phase, 2*np.pi*targ_phase)
+
+    def test_spaced_1dperiod(self):
+        events = np.array([.1, 1.1, 2.1, 4.1, 10.1])
+        period = 1
+        targ_strength = 1.
+        targ_phase = .1
+
+        strength, phase = vectorstrength(events, period)
+
+        assert_equal(strength.ndim, 0)
+        assert_equal(phase.ndim, 0)
+        assert_almost_equal(strength, targ_strength)
+        assert_almost_equal(phase, 2*np.pi*targ_phase)
+
+    def test_spaced_2dperiod(self):
+        events = np.array([.1, 1.1, 2.1, 4.1, 10.1])
+        period = [1, .5]
+        targ_strength = [1.]*2
+        targ_phase = np.array([.1, .2])
+
+        strength, phase = vectorstrength(events, period)
+
+        assert_equal(strength.ndim, 1)
+        assert_equal(phase.ndim, 1)
+        assert_almost_equal(strength, targ_strength)
+        assert_almost_equal(phase, 2*np.pi*targ_phase)
+
+    def test_partial_1dperiod(self):
+        events = np.array([.25, .5, .75])
+        period = 1
+        targ_strength = 1./3.
+        targ_phase = .5
+
+        strength, phase = vectorstrength(events, period)
+
+        assert_equal(strength.ndim, 0)
+        assert_equal(phase.ndim, 0)
+        assert_almost_equal(strength, targ_strength)
+        assert_almost_equal(phase, 2*np.pi*targ_phase)
+
+    def test_partial_2dperiod(self):
+        events = np.array([.25, .5, .75])
+        period = [1., 1., 1., 1.]
+        targ_strength = [1./3.]*4
+        targ_phase = np.array([.5, .5, .5, .5])
+
+        strength, phase = vectorstrength(events, period)
+
+        assert_equal(strength.ndim, 1)
+        assert_equal(phase.ndim, 1)
+        assert_almost_equal(strength, targ_strength)
+        assert_almost_equal(phase, 2*np.pi*targ_phase)
+
+    def test_opposite_1dperiod(self):
+        events = np.array([0, .25, .5, .75])
+        period = 1.
+        targ_strength = 0
+
+        strength, phase = vectorstrength(events, period)
+
+        assert_equal(strength.ndim, 0)
+        assert_equal(phase.ndim, 0)
+        assert_almost_equal(strength, targ_strength)
+
+    def test_opposite_2dperiod(self):
+        events = np.array([0, .25, .5, .75])
+        period = [1.]*10
+        targ_strength = [0.]*10
+
+        strength, phase = vectorstrength(events, period)
+
+        assert_equal(strength.ndim, 1)
+        assert_equal(phase.ndim, 1)
+        assert_almost_equal(strength, targ_strength)
+
+    def test_2d_events_ValueError(self):
+        events = np.array([[1, 2]])
+        period = 1.
+        assert_raises(ValueError, vectorstrength, events, period)
+
+    def test_2d_period_ValueError(self):
+        events = 1.
+        period = np.array([[1]])
+        assert_raises(ValueError, vectorstrength, events, period)
+
+    def test_zero_period_ValueError(self):
+        events = 1.
+        period = 0
+        assert_raises(ValueError, vectorstrength, events, period)
+
+    def test_negative_period_ValueError(self):
+        events = 1.
+        period = -1
+        assert_raises(ValueError, vectorstrength, events, period)
+
+
 if __name__ == "__main__":
     run_module_suite()
