@@ -10,15 +10,17 @@ import sys
 from numpy.testing import (TestCase, run_module_suite, assert_equal,
     assert_array_equal, assert_almost_equal, assert_array_almost_equal,
     assert_allclose, assert_, assert_raises, rand, dec)
-from numpy.testing.utils import WarningManager
 from nose import SkipTest
 
 import numpy
 import numpy as np
 from numpy import typecodes, array
+from scipy.lib._version import NumpyVersion
 from scipy import special
 import scipy.stats as stats
-from scipy.stats.distributions import argsreduce
+from scipy.stats._distn_infrastructure import argsreduce
+import scipy.stats.distributions
+
 from scipy.special import xlogy
 
 
@@ -26,20 +28,8 @@ from scipy.special import xlogy
 DOCSTRINGS_STRIPPED = sys.flags.optimize > 1
 
 
-def kolmogorov_check(diststr, args=(), N=20, significance=0.01):
-    qtest = stats.ksoneisf(significance, N)
-    cdf = eval('stats.'+diststr+'.cdf')
-    dist = eval('stats.'+diststr)
-    # Get random numbers
-    kwds = {'size':N}
-    vals = numpy.sort(dist.rvs(*args, **kwds))
-    cdfvals = cdf(vals, *args)
-    q = max(abs(cdfvals - np.arange(1.0, N+1)/N))
-    assert_(q < qtest, msg="Failed q=%f, bound=%f, alpha=%f" % (q, qtest, significance))
-    return
-
-
-# generate test cases to test cdf and distribution consistency
+# Generate test cases to test cdf and distribution consistency.
+# Note that this list does not include all distributions.
 dists = ['uniform','norm','lognorm','expon','beta',
          'powerlaw','bradford','burr','fisk','cauchy','halfcauchy',
          'foldcauchy','gamma','gengamma','loggamma',
@@ -51,7 +41,19 @@ dists = ['uniform','norm','lognorm','expon','beta',
          'weibull_min','weibull_max','dweibull','maxwell','rayleigh',
          'genlogistic', 'logistic','gumbel_l','gumbel_r','gompertz',
          'hypsecant', 'laplace', 'reciprocal','triang','tukeylambda',
-         'vonmises', 'pearson3']
+         'vonmises', 'vonmises_line', 'pearson3']
+
+
+def _assert_hasattr(a, b, msg=None):
+    if msg is None:
+        msg = '%s does not have attribute %s' % (a, b)
+    assert_(hasattr(a, b), msg=msg)
+
+
+def test_api_regression():
+    # https://github.com/scipy/scipy/issues/3802
+    _assert_hasattr(scipy.stats.distributions, 'f_gen')
+
 
 # check function for test generator
 
@@ -116,6 +118,11 @@ def test_vonmises_pdf_periodic():
             yield check_vonmises_cdf_periodic, k, 0, 10, x
 
 
+def test_vonmises_line_support():
+    assert_equal(stats.vonmises_line.a, -np.pi)
+    assert_equal(stats.vonmises_line.b, np.pi)
+
+
 class TestRandInt(TestCase):
     def test_rvs(self):
         vals = stats.randint.rvs(5,30,size=100)
@@ -249,11 +256,23 @@ class TestGeom(TestCase):
         assert_allclose(vals1, vals2, rtol=1e-15, atol=0)
 
     def test_cdf_sf(self):
-        vals = stats.geom.cdf([1,2,3],0.5)
-        vals_sf = stats.geom.sf([1,2,3],0.5)
-        expected = array([0.5,0.75,0.875])
-        assert_array_almost_equal(vals,expected)
-        assert_array_almost_equal(vals_sf,1-expected)
+        vals = stats.geom.cdf([1, 2, 3], 0.5)
+        vals_sf = stats.geom.sf([1, 2, 3], 0.5)
+        expected = array([0.5, 0.75, 0.875])
+        assert_array_almost_equal(vals, expected)
+        assert_array_almost_equal(vals_sf, 1-expected)
+
+    def test_logcdf_logsf(self):
+        vals = stats.geom.logcdf([1, 2, 3], 0.5)
+        vals_sf = stats.geom.logsf([1, 2, 3], 0.5)
+        expected = array([0.5, 0.75, 0.875])
+        assert_array_almost_equal(vals, np.log(expected))
+        assert_array_almost_equal(vals_sf, np.log1p(-expected))
+
+    def test_ppf(self):
+        vals = stats.geom.ppf([0.5, 0.75, 0.875], 0.5)
+        expected = array([1.0, 2.0, 3.0])
+        assert_array_almost_equal(vals, expected)
 
 
 class TestTruncnorm(TestCase):
@@ -316,8 +335,12 @@ class TestHypergeom(TestCase):
         hgpmf = stats.hypergeom.pmf(2, tot, good, N)
         assert_almost_equal(hgpmf, 0.0010114963068932233, 11)
 
+    def test_cdf_above_one(self):
+        # for some values of parameters, hypergeom cdf was >1, see gh-2238
+        assert_(0 <= stats.hypergeom.cdf(30, 13397950, 4363, 12390) <= 1.0)
+     
     def test_precision2(self):
-        """Test hypergeom precision for large numbers.  See #1218."""
+        # Test hypergeom precision for large numbers.  See #1218.
         # Results compared with those from R.
         oranges = 9.9e4
         pears = 1.1e5
@@ -505,6 +528,15 @@ class TestZipf(TestCase):
         assert_(isinstance(val, numpy.ndarray))
         assert_(val.dtype.char in typecodes['AllInteger'])
 
+    def test_moments(self):
+        # n-th moment is finite iff a > n + 1
+        m, v = stats.zipf.stats(a=2.8)
+        assert_(np.isfinite(m))
+        assert_equal(v, np.inf)
+
+        s, k = stats.zipf.stats(a=4.8, moments='sk')
+        assert_(not np.isfinite([s, k]).all())
+
 
 class TestDLaplace(TestCase):
     def test_rvs(self):
@@ -516,6 +548,8 @@ class TestDLaplace(TestCase):
         val = stats.dlaplace(1.5).rvs(3)
         assert_(isinstance(val, numpy.ndarray))
         assert_(val.dtype.char in typecodes['AllInteger'])
+        assert_(stats.dlaplace.rvs(0.8) is not None)
+
 
     def test_stats(self):
         # compare the explicit formulas w/ direct summation using pmf
@@ -538,6 +572,49 @@ class TestDLaplace(TestCase):
         assert_allclose((v, k), (4., 3.25))
 
 
+class TestInvGamma(TestCase):
+    @dec.skipif(NumpyVersion(np.__version__) < '1.7.0',
+                "assert_* funcs broken with inf/nan")
+    def test_invgamma_inf_gh_1866(self):
+        # invgamma's moments are only finite for a>n
+        # specific numbers checked w/ boost 1.54
+        with warnings.catch_warnings():
+            warnings.simplefilter('error', RuntimeWarning)
+            mvsk = stats.invgamma.stats(a=19.31, moments='mvsk')
+            assert_allclose(mvsk,
+                [0.05461496450, 0.0001723162534, 1.020362676, 2.055616582])
+
+            a = [1.1, 3.1, 5.6]
+            mvsk = stats.invgamma.stats(a=a, moments='mvsk')
+            expected = ([10., 0.476190476, 0.2173913043],       # mmm
+                        [np.inf, 0.2061430632, 0.01312749422],  # vvv
+                        [np.nan, 41.95235392, 2.919025532],     # sss
+                        [np.nan, np.nan, 24.51923076])          # kkk
+            for x, y in zip(mvsk, expected):
+                assert_almost_equal(x, y)
+
+
+class TestF(TestCase):
+    def test_f_moments(self):
+        # n-th moment of F distributions is only finite for n < dfd / 2
+        m, v, s, k = stats.f.stats(11, 6.5, moments='mvsk')
+        assert_(np.isfinite(m))
+        assert_(np.isfinite(v))
+        assert_(np.isfinite(s))
+        assert_(not np.isfinite(k))
+
+    def test_moments_warnings(self):
+        # no warnings should be generated for dfd = 2, 4, 6, 8 (div by zero)
+        with warnings.catch_warnings():
+            warnings.simplefilter('error', RuntimeWarning)
+            stats.f.stats(dfn=[11]*4, dfd=[2, 4, 6, 8], moments='mvsk')
+
+    @dec.knownfailureif(True, 'f stats does not properly broadcast')
+    def test_stats_broadcast(self):
+        # stats do not fully broadcast just yet
+        mv = stats.f.stats(dfn=11, dfd=[11, 12])
+
+
 def test_rvgeneric_std():
     # Regression test for #1191
     assert_array_almost_equal(stats.t.std([5, 6]), [1.29099445, 1.22474487])
@@ -746,6 +823,24 @@ class TestEntropy(TestCase):
         assert_almost_equal(stats.entropy([0, 1, 2]), 0.63651416829481278,
                             decimal=12)
 
+    def test_entropy_2d(self):
+        pk = [[0.1, 0.2], [0.6, 0.3], [0.3, 0.5]]
+        qk = [[0.2, 0.1], [0.3, 0.6], [0.5, 0.3]]
+        assert_array_almost_equal(stats.entropy(pk, qk),
+                [0.1933259, 0.18609809])
+
+    @dec.skipif(NumpyVersion(np.__version__) < '1.7.0',
+                "assert_* funcs broken with inf/nan")
+    def test_entropy_2d_zero(self):
+        pk = [[0.1, 0.2], [0.6, 0.3], [0.3, 0.5]]
+        qk = [[0.0, 0.1], [0.3, 0.6], [0.5, 0.3]]
+        assert_array_almost_equal(stats.entropy(pk, qk),
+                [np.inf, 0.18609809])
+
+        pk[0][0] = 0.0
+        assert_array_almost_equal(stats.entropy(pk, qk),
+                [0.17403988, 0.18609809])
+
 
 def TestArgsreduce():
     a = array([1,3,2,1,2,3,3])
@@ -822,7 +917,7 @@ class TestFitMethod(object):
             yield check, func, dist, args, alpha
 
     def test_fix_fit_2args_lognorm(self):
-        """Regression test for #1551."""
+        # Regression test for #1551.
         np.random.seed(12345)
         with np.errstate(all='ignore'):
             x = stats.lognorm.rvs(0.25, 0., 20.0, size=20)
@@ -939,11 +1034,10 @@ class TestFitMethod(object):
 
 
 class TestFrozen(TestCase):
-    """Test that a frozen distribution gives the same results as the original object.
-
-    Only tested for the normal distribution (with loc and scale specified) and for the
-    gamma distribution (with a shape parameter specified).
-    """
+    # Test that a frozen distribution gives the same results as the original object.
+    #
+    # Only tested for the normal distribution (with loc and scale specified)
+    # and for the gamma distribution (with a shape parameter specified).
     def test_norm(self):
         dist = stats.norm
         frozen = stats.norm(loc=10.0, scale=3.0)
@@ -1056,14 +1150,35 @@ class TestFrozen(TestCase):
         # the focus of this test.
         assert_equal(m1, m2)
 
+    def test_ab(self):
+        # test that the support of a frozen distribution 
+        # (i) remains frozen even if it changes for the original one
+        # (ii) is actually correct if the shape parameters are such that
+        #      the values of [a, b] are not the default [0, inf]
+        # take a genpareto as an example where the support
+        # depends on the value of the shape parameter:
+        # for c > 0: a, b = 0, inf
+        # for c < 0: a, b = 0, -1/c
+        rv = stats.genpareto(c=-0.1)
+        a, b = rv.dist.a, rv.dist.b
+        assert_equal([a, b], [0., 10.])
+
+        stats.genpareto.pdf(0, c=0.1)  # this changes genpareto.b
+        assert_equal([rv.dist.a, rv.dist.b], [a, b])
+
+        rv1 = stats.genpareto(c=0.1)
+        assert_(rv1.dist is not rv.dist)
+
+    def test_rv_frozen_in_namespace(self):
+        # Regression test for gh-3522
+        assert_(hasattr(stats.distributions, 'rv_frozen'))
+
 
 class TestExpect(TestCase):
-    """Test for expect method.
-
-    Uses normal distribution and beta distribution for finite bounds, and
-    hypergeom for discrete distribution with finite support
-
-    """
+    # Test for expect method.
+    #
+    # Uses normal distribution and beta distribution for finite bounds, and
+    # hypergeom for discrete distribution with finite support
     def test_norm(self):
         v = stats.norm.expect(lambda x: (x-5)*(x-5), loc=5, scale=2)
         assert_almost_equal(v, 4, decimal=14)
@@ -1153,6 +1268,15 @@ class TestExpect(TestCase):
         res2 = halflog.expect(args=(1.5,))
         assert_almost_equal(res1, res2, decimal=14)
 
+    def test_rice_overflow(self):
+        # rice.pdf(999, 0.74) was inf since special.i0 silentyly overflows
+        # check that using i0e fixes it
+        assert_(np.isfinite(stats.rice.pdf(999, 0.74)))
+
+        assert_(np.isfinite(stats.rice.expect(lambda x: 1, args=(0.74,))))
+        assert_(np.isfinite(stats.rice.expect(lambda x: 2, args=(0.74,))))
+        assert_(np.isfinite(stats.rice.expect(lambda x: 3, args=(0.74,))))
+
 
 class TestNct(TestCase):
     def test_nc_parameter(self):
@@ -1170,7 +1294,7 @@ class TestNct(TestCase):
                           [0.00153078, 0.00291093, 0.00525206, 0.00900815]])
         assert_allclose(res, expected, rtol=1e-5)
 
-    def text_variance_gh_issue_2401():
+    def text_variance_gh_issue_2401(self):
         # Computation of the variance of a non-central t-distribution resulted
         # in a TypeError: ufunc 'isinf' not supported for the input types,
         # and the inputs could not be safely coerced to any supported types
@@ -1178,11 +1302,90 @@ class TestNct(TestCase):
         rv = stats.nct(4, 0)
         assert_equal(rv.var(), 2.0)
 
+    def test_nct_inf_moments(self):
+        # n-th moment of nct only exists for df > n
+        m, v, s, k = stats.nct.stats(df=1.9, nc=0.3, moments='mvsk')
+        assert_(np.isfinite(m))
+        assert_equal([v, s, k], [np.inf, np.nan, np.nan])
+
+        m, v, s, k = stats.nct.stats(df=3.1, nc=0.3, moments='mvsk')
+        assert_(np.isfinite([m, v, s]).all())
+        assert_equal(k, np.nan)
+
+
+class TestRice(TestCase):
+    def test_rice_zero_b(self):
+        # rice distribution should work with b=0, cf gh-2164
+        x = [0.2, 1., 5.]
+        assert_(np.isfinite(stats.rice.pdf(x, b=0.)).all())
+        assert_(np.isfinite(stats.rice.logpdf(x, b=0.)).all())
+        assert_(np.isfinite(stats.rice.cdf(x, b=0.)).all())
+        assert_(np.isfinite(stats.rice.logcdf(x, b=0.)).all())
+
+        q = [0.1, 0.1, 0.5, 0.9]
+        assert_(np.isfinite(stats.rice.ppf(q, b=0.)).all())
+
+        mvsk = stats.rice.stats(0, moments='mvsk')
+        assert_(np.isfinite(mvsk).all())
+
+        # furthermore, pdf is continuous as b\to 0
+        # rice.pdf(x, b\to 0) = x exp(-x^2/2) + O(b^2)
+        # see e.g. Abramovich & Stegun 9.6.7 & 9.6.10
+        b = 1e-8
+        assert_allclose(stats.rice.pdf(x, 0), stats.rice.pdf(x, b),
+                atol=b, rtol=0)
+
+    def test_rice_rvs(self):
+        rvs = stats.rice.rvs
+        assert_equal(rvs(b=3.).size, 1)
+        assert_equal(rvs(b=3., size=(3, 5)).shape, (3, 5))
+
+
+class TestErlang(TestCase):
+    def test_erlang_runtimewarning(self):
+        # erlang should generate a RuntimeWarning if a non-integer
+        # shape parameter is used.
+        with warnings.catch_warnings():
+            warnings.simplefilter("error", RuntimeWarning)
+
+            # The non-integer shape parameter 1.3 should trigger a RuntimeWarning
+            assert_raises(RuntimeWarning,
+                              stats.erlang.rvs, 1.3, loc=0, scale=1, size=4)
+
+            # Calling the fit method with `f0` set to an integer should
+            # *not* trigger a RuntimeWarning.  It should return the same
+            # values as gamma.fit(...).
+            data = [0.5, 1.0, 2.0, 4.0]
+            result_erlang = stats.erlang.fit(data, f0=1)
+            result_gamma = stats.gamma.fit(data, f0=1)
+            assert_allclose(result_erlang, result_gamma, rtol=1e-3)
+
+
+class TestRdist(TestCase):
+    @dec.slow
+    def test_rdist_cdf_gh1285(self):
+        # check workaround in rdist._cdf for issue gh-1285.
+        distfn = stats.rdist
+        values = [0.001, 0.5, 0.999]
+        assert_almost_equal(distfn.cdf(distfn.ppf(values, 541.0), 541.0),
+                                values, decimal=5)
+
+
+def test_540_567():
+    # test for nan returned in tickets 540, 567
+    assert_almost_equal(stats.norm.cdf(-1.7624320982),0.03899815971089126,
+                            decimal=10, err_msg='test_540_567')
+    assert_almost_equal(stats.norm.cdf(-1.7624320983),0.038998159702449846,
+                            decimal=10, err_msg='test_540_567')
+    assert_almost_equal(stats.norm.cdf(1.38629436112, loc=0.950273420309,
+                            scale=0.204423758009),0.98353464004309321,
+                            decimal=10, err_msg='test_540_567')
+
 
 def test_regression_ticket_1316():
     # The following was raising an exception, because _construct_default_doc()
     # did not handle the default keyword extradoc=None.  See ticket #1316.
-    g = stats.distributions.gamma_gen(name='gamma')
+    g = stats._continuous_distns.gamma_gen(name='gamma')
 
 
 def test_regression_ticket_1326():
@@ -1355,15 +1558,13 @@ def test_ksone_fit_freeze():
          -0.10943985, -0.35243174, 0.06897665, -0.03553363, -0.0701746,
          -0.06037974, 0.37670779, -0.21684405])
 
-    olderr = np.seterr(invalid='ignore')
-    warn_ctx = WarningManager()
-    warn_ctx.__enter__()
     try:
-        warnings.simplefilter('ignore', UserWarning)
-        warnings.simplefilter('ignore', RuntimeWarning)
-        stats.ksone.fit(d)
+        olderr = np.seterr(invalid='ignore')
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore', UserWarning)
+            warnings.simplefilter('ignore', RuntimeWarning)
+            stats.ksone.fit(d)
     finally:
-        warn_ctx.__exit__()
         np.seterr(**olderr)
 
 
@@ -1445,7 +1646,7 @@ def test_ncx2_tails_ticket_955():
     # Trac #955 -- check that the cdf computed by special functions
     # matches the integrated pdf
     a = stats.ncx2.cdf(np.arange(20, 25, 0.2), 2, 1.07458615e+02)
-    b = stats.ncx2.veccdf(np.arange(20, 25, 0.2), 2, 1.07458615e+02)
+    b = stats.ncx2._cdfvec(np.arange(20, 25, 0.2), 2, 1.07458615e+02)
     assert_allclose(a, b, rtol=1e-3, atol=0)
 
 
@@ -1455,6 +1656,24 @@ def test_foldnorm_zero():
     assert_equal(rv.cdf(0), 0)  # rv.cdf(0) previously resulted in: nan
 
 
+def test_stats_shapes_argcheck():
+    # stats method was failing for vector shapes if some of the values
+    # were outside of the allowed range, see gh-2678
+    mv3 = stats.invgamma.stats([0.0, 0.5, 1.0], 1, 0.5)  # 0 is not a legal `a`
+    mv2 = stats.invgamma.stats([0.5, 1.0], 1, 0.5)
+    mv2_augmented = tuple(np.r_[np.nan, _] for _ in mv2)
+    assert_equal(mv2_augmented, mv3)
+
+    mv3 = stats.lognorm.stats([2, 2.4, -1])  # -1 is not a legal shape parameter
+    mv2 = stats.lognorm.stats([2, 2.4])
+    mv2_augmented = tuple(np.r_[_, np.nan] for _ in mv2)
+    assert_equal(mv2_augmented, mv3)
+
+    # FIXME: this is only a quick-and-dirty test of a quick-and-dirty bugfix.
+    # stats method with multiple shape parameters is not properly vectorized
+    # anyway, so some distributions may or may not fail.
+
+
 ## Test subclassing distributions w/ explicit shapes
 
 class _distr_gen(stats.rv_continuous):
@@ -1472,13 +1691,13 @@ class _distr3_gen(stats.rv_continuous):
         return a + b
 
     def _cdf(self, x, a):
-        """Different # of shape params from _pdf, to be able to check that
-        inspection catches the inconsistency."""
+        # Different # of shape params from _pdf, to be able to check that
+        # inspection catches the inconsistency."""
         return 42 * a + x
 
 
 class _distr6_gen(stats.rv_continuous):
-    #Two shape parameters (both _pdf and _cdf defined, consistent shapes.)
+    # Two shape parameters (both _pdf and _cdf defined, consistent shapes.)
     def _pdf(self, x, a, b):
         return a*x + b
 
@@ -1487,7 +1706,7 @@ class _distr6_gen(stats.rv_continuous):
 
 
 class TestSubclassingExplicitShapes(TestCase):
-    """Construct a distribution w/ explicit shapes parameter and test it."""
+    # Construct a distribution w/ explicit shapes parameter and test it.
 
     def test_correct_shapes(self):
         dummy_distr = _distr_gen(name='dummy', shapes='a')
@@ -1499,7 +1718,7 @@ class TestSubclassingExplicitShapes(TestCase):
 
     def test_wrong_shapes_2(self):
         dummy_distr = _distr_gen(name='dummy', shapes='a, b, c')
-        dct =dict(a=1, b=2, c=3)
+        dct = dict(a=1, b=2, c=3)
         assert_raises(TypeError, dummy_distr.pdf, 1, **dct)
 
     def test_shapes_string(self):
@@ -1537,7 +1756,6 @@ class TestSubclassingExplicitShapes(TestCase):
         dct = dict(name='dummy', shapes='a, b, c, lambda')
         assert_raises(SyntaxError, _distr_gen, **dct)
 
-
     def test_shapes_signature(self):
         # test explicit shapes which agree w/ the signature of _pdf
         class _dist_gen(stats.rv_continuous):
@@ -1547,7 +1765,6 @@ class TestSubclassingExplicitShapes(TestCase):
         dist = _dist_gen(shapes='a')
         assert_equal(dist.pdf(0.5, a=2), stats.norm.pdf(0.5)*2)
 
-
     def test_shapes_signature_inconsistent(self):
         # test explicit shapes which do not agree w/ the signature of _pdf
         class _dist_gen(stats.rv_continuous):
@@ -1557,7 +1774,6 @@ class TestSubclassingExplicitShapes(TestCase):
         dist = _dist_gen(shapes='a, b')
         assert_raises(TypeError, dist.pdf, 0.5, **dict(a=1, b=2))
 
-
     def test_star_args(self):
         # test _pdf with only starargs
         # NB: **kwargs of pdf will never reach _pdf
@@ -1598,7 +1814,6 @@ class TestSubclassingExplicitShapes(TestCase):
         dist = _distr_gen(shapes='extra_kwarg')
         assert_equal(dist.pdf(1, extra_kwarg=3), stats.norm.pdf(1))
 
-
     def shapes_empty_string(self):
         # shapes='' is equivalent to shapes=None
         class _dist_gen(stats.rv_continuous):
@@ -1610,7 +1825,7 @@ class TestSubclassingExplicitShapes(TestCase):
 
 
 class TestSubclassingNoShapes(TestCase):
-    """Construct a distribution w/o explicit shapes parameter and test it."""
+    # Construct a distribution w/o explicit shapes parameter and test it.
 
     def test_only__pdf(self):
         dummy_distr = _distr_gen(name='dummy')
@@ -1680,7 +1895,12 @@ def test_docstrings():
         dist = getattr(stats, distname)
         if isinstance(dist, (stats.rv_discrete, stats.rv_continuous)):
             for regex in badones:
-                assert_( re.search(regex, dist.__doc__) is None)
+                assert_(re.search(regex, dist.__doc__) is None)
+
+
+def test_infinite_input():
+    assert_almost_equal(stats.skellam.sf(np.inf, 10, 11), 0)
+    assert_almost_equal(stats.ncx2._cdf(np.inf, 8, 0.1), 1)
 
 
 if __name__ == "__main__":
